resource "aws_kinesis_firehose_delivery_stream" "data_stream" {
  name        = var.kinesis_firehose_name
  destination = "extended_s3"

  extended_s3_configuration {
    role_arn            = var.iam_role_arn
    bucket_arn          = var.destination_bucket_arn
    prefix              = var.stream_prefix
    error_output_prefix = var.error_output_prefix
    buffer_size         = var.buffer_size
    buffer_interval     = var.buffer_interval
    compression_format  = var.compression_format

    cloudwatch_logging_options {
      enabled         = var.cloudwatch_logging_enabled
      log_group_name  = var.cloudwatch_logging_log_group_name
      log_stream_name = var.cloudwatch_logging_log_stream_name
    }

    dynamic_partitioning_configuration {
      enabled        = var.dynamic_partitioning_enabled
      retry_duration = var.dynamic_partitioning_retry_duration
    }

    processing_configuration {
      enabled = var.dynamic_partitioning_enabled

      # JQ processor example
      dynamic "processors" {
        for_each = var.metadata_extraction_enabled ? ["true"] : []
        content {
          type = "MetadataExtraction"
          parameters {
            parameter_name  = "JsonParsingEngine"
            parameter_value = "JQ-1.6"
          }
          parameters {
            parameter_name  = "MetadataExtractionQuery"
            parameter_value = var.metadata_extraction_query
          }
        }
      }

      # Lambda processor example
      dynamic "processors" {
        for_each = var.lambda_processor_enabled ? ["true"] : []
        content {
          type = "Lambda"
          parameters {
            parameter_name  = "LambdaArn"
            parameter_value = "${var.lambda_processor_arn}:${var.lambda_version}"
          }
        }
      }
    }

    dynamic "data_format_conversion_configuration" {
      for_each = var.data_format_conversion_enabled ? ["true"] : []
      content {
        input_format_configuration {
          deserializer {
            dynamic "hive_json_ser_de" {
              for_each = var.deserializer == "hive_json_ser_de" ? ["true"] : []
              content {}
            }
            dynamic "open_x_json_ser_de" {
              for_each = var.deserializer == "open_x_json_ser_de" ? ["true"] : []
              content {}
            }
          }
        }

        output_format_configuration {
          serializer {
            dynamic "orc_ser_de" {
              for_each = var.serializer == "orc_ser_de" ? ["true"] : []
              content {}
            }
            dynamic "parquet_ser_de" {
              for_each = var.serializer == "parquet_ser_de" ? ["true"] : []
              content {}
            }
          }
        }

        schema_configuration {
          database_name = var.schema_configuration_database_name
          role_arn      = var.schema_configuration_role_arn
          table_name    = var.schema_configuration_table_name
        }
      }
    }

  }

  dynamic "kinesis_source_configuration" {
    for_each = length(var.source_kinesis_arn) > 0 && length(var.iam_role_arn) > 0 ? ["true"] : []
    content {
      role_arn           = var.iam_role_arn
      kinesis_stream_arn = var.source_kinesis_arn
    }
  }

  tags = var.tags
}

resource "aws_iam_policy" "firehose-put-records-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-firehose-put-records"
  path        = "/"
  description = "Kinesis Firehose policy to allow Put Records"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "firehose:PutRecord",
        "firehose:PutRecordBatch"
      ],
      "Effect": "Allow",
      "Resource": ["${aws_kinesis_firehose_delivery_stream.data_stream.arn}"]
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }

  tags = var.tags
}